/* libiir/src/libiir++/200_Filter.h
 * 
 *  Copyright: ©2014, Laurence Withers.
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
 */



/*! \defgroup libiirpp_filter Basic IIR filtering

\ingroup libiirpp

An instance of the \ref Filter class represents the structure and state of an
%IIR filter. It may be built up with \ref Coeff instances or parsed from a
user-supplied string (see \ref string_desc).

*/



/*! \brief %IIR filter coefficient set

\ingroup libiirpp_filter

An instance of this class contains both a copy of the set of coefficients used
in each filter stage (of which there may be multiple), and all of the state
(i.e.  previous inputs and outputs) associated with each stage.

*/
class Filter {
public:
    /*! \brief Construct from coefficient set.

    \param coeff Coefficient set.

    This constructor builds a filter chain with one stage. Additional stages may
    be added via \ref chainCoeffSet(). This constructor will always result in a
    valid object.

    */
    Filter(const Coeff& coeff);

    /*! \brief Copy constructor.

    \param other Filter to copy.

    Copies the structure (but not the state) of the filter \a other. This
    constructor will always result in a valid object.

    */
    Filter(const Filter& other);

    /*! \brief Construct from filter description string.

    \param desc Description string.

    This constructor will parse the description string \a desc (see
    \ref string_desc for details on what is accepted) and build up a filter
    as described.

    If the parsing step fails, then \ref valid() will return false and the
    object must not be used (no other functions except the destructor will be
    safe to call).

    */
    Filter(const char* desc);

    /*! \brief Destructor. */
    virtual ~Filter();



    /*! \brief Determine whether object is valid.

    \retval true if object is valid.
    \retval false if object is not valid.

    A Filter object might not be valid if it was constructed from a description
    string that contained an error. In that case this function will return
    \a false and the object should not be used (no other functions except the
    destructor will be safe to call).

    */
    bool valid() const;



    /*! \brief %Filter a sample.

    \param x Sample.
    \returns %Filter output.

    Injects a single sample into the filter and returns the output value.

    */
    double f(double x);



    /*! \brief Retrieve number of coefficient sets.

    \returns Number of sets in use (≥1).

    */
    int numCoeffSet() const;

    /*! \brief Retrieve coefficient set.

    \param idx Index of set (0 ≤ \a idx < \ref numCoeffSet()).
    \returns Reference to coefficient set.

    */
    const Coeff& getCoeffSet(int idx) const;

    /*! \brief Add another set of coefficients to the filter chain.

    \param coeff Coefficient set to add.

    */
    void chainCoeffSet(const Coeff& coeff);



    /*! \brief Evaluate response of filter at given frequency.

    \param freq Frequency (0 ≤ \a freq ≤ 1).
    \returns Complex response of filter at given frequency.

    Returns the steady-state response of the filter to a pure input signal of
    the given frequency. \a freq is expressed as a fraction of the intended
    sampling rate (with 0.5 being the Nyquist frequency).

    */
    std::complex<double> response(double freq) const;



    /*! \brief Search for -3dB points.

    \param gain Nominal gain of filter.
    \returns List of frequency ratios at which filter magnitude is -3dB.

    This function will perform a linear scan of the frequency response of
    the filter between DC and the Nyquist frequency. It will then search
    for any pair of points between which the magnitude transitions the -3dB
    level. It then performs a few levels of binary search in order to provide
    a more accurate answer.

    Returned frequencies are expressed as a fraction of the sampling rate.

    */
    std::vector<double> responseMinus3dB(double gain) const;

private:
    // one entry per set of coefficients, in order of execution
    std::vector<const Coeff*> coeff_;

    // the C object has both state and its own copy of the coefficients
    struct iir_filter_t* fi_;

    // disallow assignment
    Filter& operator=(const Filter& other);
};



/* options for text editors
vim: expandtab:ts=4:sw=4:syntax=cpp.doxygen
*/
