/* libiir/src/libiir++/300_common_filters.h
 * 
 *  Copyright: ©2014, Laurence Withers.
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
 */



/*! \defgroup libiirpp_common_filters Common types of IIR filter

\ingroup libiirpp

Routines in this section can be used to build a coefficient set that matches a
common %IIR filter type. These can be passed to the \ref Filter class to build
up an actual instance of a filter.

\note It is common to build higher-order filters by chaining together several
    lower-order filters. The functions in this section do not do that directly,
    but are intended to build the individual components of a chain. See
    \ref Filter::Filter(const char*) for an interface which performs this
    automatically.

*/
/*!@{*/



/*! \brief nth-order Butterworth low-pass

\param order Order of filter (≥1).
\param gain Linear gain of filter.
\param corner Corner frequency expressed as a fraction of Nyquist
    (0 ≤ \a corner ≤ 1).

\returns %IIR filter coefficient set.

Uses the Exstrom labs code to compute the coefficients of an nth-order (param
\a order) Butterworth-type low pass filter with gain \a gain and corner
frequency \a corner.

Note it is recommended to chain multiple filters together to build anything
greater than a 4th-order filter. This function won't do that directly for you.
\a gain will usually be set to be 1.0.

The corner frequency corner is expressed as a fraction of the sampling
frequency (which is of course not known by the %IIR code). It should lie between
0 (0Hz) and 1 (the Nyquist frequency, or ½ the sampling frequency). 

*/
Coeff ButterworthLowPass(int order, double gain, double corner);



/*! \brief nth-order Butterworth high-pass

\param order Order of filter (≥1).
\param gain Linear gain of filter.
\param corner Corner frequency expressed as a fraction of Nyquist
    (0 ≤ \a corner ≤ 1).

\returns %IIR filter coefficient set.

Uses the Exstrom labs code to compute the coefficients of an nth-order (param
\a order) Butterworth-type high pass filter with gain \a gain and corner
frequency \a corner.

Note it is recommended to chain multiple filters together to build anything
greater than a 4th-order filter. This function won't do that directly for you.
\a gain will usually be set to be 1.0.

The corner frequency corner is expressed as a fraction of the sampling
frequency (which is of course not known by the %IIR code). It should lie between
0 (0Hz) and 1 (the Nyquist frequency, or ½ the sampling frequency). 

*/
Coeff ButterworthHighPass(int order, double gain, double corner);



/*! \brief nth-order Butterworth band-pass

\param order Order of filter (≥1).
\param gain Linear gain of filter.
\param corner1 Low corner frequency expressed as a fraction of Nyquist
    (0 ≤ \a corner ≤ 1).
\param corner2 High corner frequency expressed as a fraction of Nyquist
    (0 ≤ \a corner ≤ 1).

\returns %IIR filter coefficient set.

Uses the Exstrom labs code to compute the coefficients of an nth-order (param
\a order) Butterworth-type band pass filter with gain \a gain and corner
frequencies \a corner1 and \a corner2.

Note it is recommended to chain multiple filters together to build anything
greater than a 4th-order filter. This function won't do that directly for you.
\a gain will usually be set to be 1.0.

The corner frequencies \a corner1 and \a corner2 are expressed as a fraction of
the sampling frequency (which is of course not known by the %IIR code). They
should lie between 0 (0Hz) and 1 (the Nyquist frequency, or ½ the sampling
frequency), and \a corner2 should be greater than \a corner1.

*/
Coeff ButterworthBandPass(int order, double gain, double corner1,
    double corner2);



/*! \brief nth-order Butterworth band-stop

\param order Order of filter (≥1).
\param gain Linear gain of filter.
\param corner1 Low corner frequency expressed as a fraction of Nyquist
    (0 ≤ \a corner ≤ 1).
\param corner2 High corner frequency expressed as a fraction of Nyquist
    (0 ≤ \a corner ≤ 1).

\returns %IIR filter coefficient set.

Uses the Exstrom labs code to compute the coefficients of an nth-order (param
\a order) Butterworth-type band stop filter with gain \a gain and corner
frequencies \a corner1 and \a corner2.

Note it is recommended to chain multiple filters together to build anything
greater than a 4th-order filter. This function won't do that directly for you.
\a gain will usually be set to be 1.0.

The corner frequencies \a corner1 and \a corner2 are expressed as a fraction of
the sampling frequency (which is of course not known by the %IIR code). They
should lie between 0 (0Hz) and 1 (the Nyquist frequency, or ½ the sampling
frequency), and \a corner2 should be greater than \a corner1.

*/
Coeff ButterworthBandStop(int order, double gain, double corner1,
    double corner2);



/*!@}*/
/* options for text editors
vim: expandtab:ts=4:sw=4:syntax=cpp.doxygen
*/
