/* libiir/src/libiir/100_coeff.c
 * 
 *  Copyright: ©2010–2014, Laurence Withers.
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
*/



/* struct iir_coeff_t
 *  Holds a general IIR filter (i.e. the set of coefficients that define it).
 *  nc >= 1 and nd >= 1.
 */
struct iir_coeff_t {
    int nc, nd;
    double* c, * d;
};



/* iir_coeff_new()
 *  Allocates a new set of coefficient objects.
 */
struct iir_coeff_t*
iir_coeff_new(int nc, const double* c, int nd, const double* d)
{
    struct iir_coeff_t* coeff;

    if(nc < 1 || nd < 1) {
        errno = EINVAL;
        return 0;
    }

    coeff = malloc(sizeof(struct iir_coeff_t));
    coeff->nc = nc;
    coeff->nd = nd;
    coeff->c = malloc(sizeof(double) * nc);
    coeff->d = malloc(sizeof(double) * nd);
    memcpy(coeff->c, c, sizeof(double) * nc);
    memcpy(coeff->d, d, sizeof(double) * nd);

    return coeff;
}



/* iir_coeff_copy()
 *  Deep copy of coefficient object.
 */
struct iir_coeff_t*
iir_coeff_copy(const struct iir_coeff_t* other)
{
    struct iir_coeff_t* coeff;

    coeff = malloc(sizeof(struct iir_coeff_t));
    coeff->nc = other->nc;
    coeff->nd = other->nd;
    coeff->c = malloc(sizeof(double) * coeff->nc);
    coeff->d = malloc(sizeof(double) * coeff->nd);
    memcpy(coeff->c, other->c, sizeof(double) * coeff->nc);
    memcpy(coeff->d, other->d, sizeof(double) * coeff->nd);

    return coeff;
}



/* iir_coeff_free()
 *  Frees memory associated with ‘coeff’.
 */
void
iir_coeff_free(struct iir_coeff_t* coeff)
{
    if(!coeff) return;
    free(coeff->c);
    free(coeff->d);
    free(coeff);
}



/* iir_coeff_get_*()
 *  Functions for querying the coefficients that make up an IIR filter.
 */
int
iir_coeff_get_nc(const struct iir_coeff_t* coeff)
{
    return coeff->nc;
}

int
iir_coeff_get_nd(const struct iir_coeff_t* coeff)
{
    return coeff->nd;
}

double
iir_coeff_get_c(const struct iir_coeff_t* coeff, int idx)
{
    return coeff->c[idx];
}

double
iir_coeff_get_d(const struct iir_coeff_t* coeff, int idx)
{
    return coeff->d[idx];
}



/* options for text editors
vim: expandtab:ts=4:sw=4
*/
