/* libiir/src/libiir/100_coeff.h
 * 
 *  Copyright: ©2010–2014, Laurence Withers.
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
*/



/*! \defgroup libiir_coeff Coefficient handling

\ingroup libiir

The functions in this module present an interface for managing the set of
coefficients of a single %IIR filter instance. See \ref iir_structure for
the definition of a coefficient set. The coefficient sets form the
basic building block of an %IIR filter instance (see \ref libiir_filter),
which may chain several %IIR filters in sequence.

*/
/*!@{*/



/* opaque structure */
struct iir_coeff_t;



/*! \brief Create %IIR coefficient set

\param nc Number of \a c coefficients.
\param c Array of \a c coefficients.
\param nd Number of \a d coefficients.
\param d Array of \a d coefficients.
\returns Pointer to new general %IIR filter object.

This function creates a new set of %IIR filter coefficients which may be used to
create filter instances through \ref iir_filter_new() or chained on to existing
instances through \ref iir_filter().

See \ref iir_structure for a full explanation of the parameters.

*/
struct iir_coeff_t* iir_coeff_new(int nc, const double* c, int nd,
    const double* d)
#ifndef DOXYGEN
    __attribute__((malloc,nonnull))
#endif
;



/*! \brief Create copy of a set of %IIR coefficients

\param other Set of %IIR filter coefficients to copy.
\returns Pointer to new general %IIR filter object.

Performs a deep copy of the set of %IIR coefficients contained within \a other.

*/
struct iir_coeff_t* iir_coeff_copy(const struct iir_coeff_t* other);



/*! \brief Free %IIR coefficient set

\param coeff Pointer to %IIR coefficient object. May be 0.

Frees a set of %IIR filter coefficients previously allocated through
\ref iir_coeff_new(). Can be called on a null pointer without consequences.
Note that \ref iir_filter_new() and \ref iir_filter_chain() actually store a
copy of the coefficients, so it is possible to free \a coeff even if existing
filters are still using its coefficient values.

*/
void iir_coeff_free(struct iir_coeff_t* coeff);



/*! \brief Query number of C coefficients.

\param coeff Pointer to %IIR coefficient object.
\returns Number of C coefficients (≥1).

*/
int iir_coeff_get_nc(const struct iir_coeff_t* coeff);



/*! \brief Query number of D coefficients.

\param coeff Pointer to %IIR coefficient object.
\returns Number of D coefficients (≥1).

*/
int iir_coeff_get_nd(const struct iir_coeff_t* coeff);



/*! \brief Get value of C coefficient.

\param coeff Pointer to %IIR coefficient object.
\param idx Index of coefficient (≥0, < \ref iir_coeff_get_nc()).
\returns C coefficient value.

*/
double iir_coeff_get_c(const struct iir_coeff_t* coeff, int idx);



/*! \brief Get value of D coefficient.

\param coeff Pointer to %IIR coefficient object.
\param idx Index of coefficient (≥0, < \ref iir_coeff_get_nd()).
\returns D coefficient value.

*/
double iir_coeff_get_d(const struct iir_coeff_t* coeff, int idx);



/*!@}*/
/* options for text editors
vim: expandtab:ts=4:sw=4:syntax=c.doxygen
*/
