/* libiir/src/libiir/200_filter.h
 * 
 *  Copyright: ©2010–2014, Laurence Withers.
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
*/



/*! \defgroup libiir_filter Basic IIR filtering

\ingroup libiir

The functions in this module present a basic interface for representing %IIR
filters, creating instances of (possibly chained) %IIR filters, and filtering an
input sample.

A general %IIR filter consists of a set of coefficients, and may be created
through \ref iir_coeff_new(). The filter object (the opaque <code>struct
iir_coeff_t</code>) is then used to instantiate specific filters (the opaque
<code>struct iir_filter_t</code>) through \ref iir_filter_new(). Each filter
instance may have an arbitrary further number of %IIR filters chained on to it
through \ref iir_filter_chain(). The filter processes one sample at a time
through \ref iir_filter().

*/
/*!@{*/



/* opaque structure */
struct iir_filter_t;



/*! \brief Create %IIR filter instance

\param coeff Filter coefficients to use.
\returns Pointer to new instance of %IIR filter.

Creates a new instance of a general %IIR filter. The set of coefficients \a coeff
is copied into the returned structure, meaning the coefficients can be freed
after this function returns if they will not be needed again.

The first sample passed through \ref iir_filter() will be used to set initial
conditions (see \ref iir_structure). 

An arbitrary number of further filters may be chained on to the end of this
instance through \ref iir_filter_chain().

*/
struct iir_filter_t* iir_filter_new(const struct iir_coeff_t* coeff)
#ifndef DOXYGEN
    __attribute__((malloc,nonnull))
#endif
;



/*! \brief Free %IIR filter instance

\param fi Filter object to free. May be 0.

Frees a previously-allocated %IIR filter instance. Can be called on a null
pointer without consequences.

*/
void iir_filter_free(struct iir_filter_t* fi);



/*! \brief Add a further %IIR filter to a filter instance

\param fi Filter instance to chain onto.
\param coeff New %IIR filter coefficients to add to chain.

Extends an existing %IIR filter by chaining a new set of coefficients onto the
end. This can be used for &gt;4th order Butterworth filters, for example. This
copies the set of coefficients from \a coeff so the coefficients can be freed
after this function returns if they are no longer required.

*/
void iir_filter_chain(struct iir_filter_t* fi, const struct iir_coeff_t* coeff)
#ifndef DOXYGEN
    __attribute__((nonnull))
#endif
;



/*! \brief Create a deep copy of an %IIR filter instance

\param fi Filter instance to copy.
\param state Non-zero to copy state as well.
\returns Pointer to newly-allocated filter instance.

Performs a deep copy of the filter instance \a fi. If \a state is non-zero,
then the internal state of \a fi is copied as well (otherwise it is as treated
as a brand new instance).

*/
struct iir_filter_t* iir_filter_copy(const struct iir_filter_t* fi, int state)
#ifndef DOXYGEN
    __attribute__((malloc,nonnull))
#endif
;



/*! \brief Process a sample

\param fi Filter object to run.
\param samp Input sample \c x(t).
\returns Filtered output sample \c y(t).

Given the input sample \c x(t) (the parameter \a samp), runs the filter chain in
\a fi and produces the output sample \c y(t), which it returns.

*/
double iir_filter(struct iir_filter_t* fi, double samp);



/*! \brief Count number of coefficient sets in %IIR filter chain

\param fi Filter object.
\returns Number of coefficient sets in chain (≥1).

Returns the number of discrete %IIR filter coefficient sets used in the filter
object \a fi. There will always be at least one set.

*/
int iir_filter_coeff_sets(const struct iir_filter_t* fi);



/*! \brief Get %IIR coefficient set from filter chain

\param fi Filter object.
\param idx Index of coefficient set (0 ≤ \a idx &lt; \ref iir_filter_coeff_sets()).
\returns Newly-allocated coefficient set object.

Extracts the %IIR filter coefficient set for the given step (\a idx) in the chain
of filters in \a fi. Returns a newly-allocated filter coefficient set object
which can be freed with \ref iir_coeff_free().

*/
struct iir_coeff_t* iir_filter_coeff_set(const struct iir_filter_t* fi,
    int idx);



/*!@}*/
/* options for text editors
vim: expandtab:ts=4:sw=4:syntax=c.doxygen
*/
