/* libiir/src/libiir/500_response.c
 * 
 *  Copyright: ©2011, Laurence Withers.
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
*/



static double complex
iir_response_aux(int nc, const double* c, int nd, const double* d, double freq)
{
    int i;
    double complex sum, resp, N;

    /* Z transform; substitutions:
     *
     * z = exp(jω)
     *   where j is the imaginary unit (written as I in the C code below)
     *   and where ω = 2πf
     *
     * z^-n = exp(jωn)
     *
     * exp(jx) = cos(x) + j.sin(x)
     *
     * z^-n = cos(-ωn) + j.sin(-ωn)
     *
     * N = -ω = -2πf
     */
    N = -2 * M_PI * freq;

    sum = 0;
    for(i = 0; i < nc; ++i) {
        sum += c[i] * (cos(i*N) + I * sin(i*N));
    }

    resp = sum;
    sum = 1;
    for(i = 1; i <= nd; ++i) {
        sum += d[i - 1] * (cos(i*N) + I * sin(i*N));
    }

    return resp / sum;
}



double complex
iir_response_c(const struct iir_coeff_t* coeff, double freq)
{
    return iir_response_aux(coeff->nc, coeff->c, coeff->nd, coeff->d, freq);
}



double complex
iir_response(const struct iir_filter_t* fi, double freq)
{
    double complex resp;

    for(resp = 1.0; fi; fi = fi->next) {
        resp *= iir_response_aux(fi->nc, fi->c, fi->nd, fi->d, freq);
    }
    return resp;
}



void
iir_response_minus_3dB_aux(const struct iir_filter_t* fi, double gain,
    double** freqs, int* num_freqs, int* sz_freqs, double f1, double f2)
{
    double f3;
    double mag;
    int i;

    for(i = 0; i < 20; ++i) {
        f3 = (f1 + f2) / 2;
        mag = cabs(iir_response(fi, f3)) / gain;
        if(mag < 0.5) {
            f1 = f3;
        } else {
            f2 = f3;
        }
    }

    if(*num_freqs == *sz_freqs) {
        *sz_freqs = (*sz_freqs) ? (*sz_freqs << 1) : 4;
        *freqs = realloc(*freqs, sizeof(double) * (*sz_freqs));
    }
    (*freqs)[*num_freqs] = f3;
    ++(*num_freqs);
}

void
iir_response_minus_3dB(const struct iir_filter_t* fi, double gain,
    double** freqs, int* num_freqs)
{
    int sz_freqs = 0, pos;
    double mag, last_mag;

    *freqs = 0;
    *num_freqs = 0;

    last_mag = cabs(iir_response(fi, 0)) / gain;
    for(pos = 1; pos < 1000; ++pos) {
        mag = cabs(iir_response(fi, 0.0005 * pos)) / gain;
        if(mag < 0.5) {
            if(last_mag >= 0.5) {
                iir_response_minus_3dB_aux(fi, gain, freqs, num_freqs,
                    &sz_freqs, 0.0005 * pos, 0.0005 * (pos - 1));
            }
        } else {
            if(last_mag < 0.5) {
                iir_response_minus_3dB_aux(fi, gain, freqs, num_freqs,
                    &sz_freqs, 0.0005 * (pos - 1), 0.0005 * pos);
            }
        }
        last_mag = mag;
    }
}



/* options for text editors
vim: expandtab:ts=4:sw=4
*/
