/* libiir/src/libiir/500_response.h
 * 
 *  Copyright: ©2011–2014, Laurence Withers.
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
*/



/*! \defgroup iir_response Measuring IIR response

\ingroup libiir

These functions allow measuring the %IIR response (i.e. frequency and phase of
the transfer function) at a given frequency. The response is a complex number;
see \c complex(5) for details. The magnitude \c cabs(z) is the gain of the
filter and the phase \c carg(z) its phase delay.

*/
/*!@{*/



/*! \brief Find response of coefficients at a single frequency.

\param coeff Set of coefficients.
\param freq Frequency of interest, as a fraction of sample rate.
\returns Response as complex number.

*/
double complex iir_response_c(const struct iir_coeff_t* coeff, double freq);



/*! \brief Find response of filter at a single frequency.

\param fi Filter to evaluate.
\param freq Frequency of interest, as a fraction of sample rate.
\returns Response as a complex number.

Similar to \ref iir_response_c(), except that it works on an instantiated
filter (and takes all parts of a chained filter into account). Does not alter
the state of the filter in any way and calls may be interleaved with filter
evaluation.

*/
double complex iir_response(const struct iir_filter_t* fi, double freq);



/*! \brief Search for -3dB points.

\param fi Filter to evaluate.
\param gain Nominal gain of the filter.
\param[out] freqs Set to point to array of frequencies
\param[out] num_freqs Number of frequencies in \a freq.

This function will perform a linear scan of the frequency response of the
filter between DC and the Nyquist frequency. It will then search for any pair
of points between which the magnitude transitions the -3dB level. It then
performs a few levels of binary search in order to provide a more accurate
answer.

Returned frequencies are expressed as a fraction of the sampling rate. The
function will allocate an array of \c double with \c malloc(3), so the result
should later be passed to \c free(3).

*/
void iir_response_minus_3dB(const struct iir_filter_t* fi, double gain,
    double** freqs, int* num_freqs);



/*!@}*/
/* options for text editors
vim: expandtab:ts=4:sw=4:syntax=c.doxygen
*/
